
import {height
  Line3,
  Plane,
  Quaternion,
  Raycaster,
  Vector2,
  Vector3,
  PerspectiveCamera,
  MathUtils
} from './three.module.js'
import {
  buildCamera,
  generateCurveBasedOnCircle,
  NDCToScreen,
  getNDCFromMouse
} from './share.js'

function getCenter(
                   groundPitch, //地面的上下摆动
                   groundYaw,    //地面的左右摆动 固定为0
                   configType, // 1配置一 2配置二
                   windowWidth,
                   windowHeight
                   )
{

    var config = secondCameraConfig;
    config.previewWindow.width = windowWidth;
    config.previewWindow.height = windowHeight;

    const camera = buildCamera(config)
    const plane = new Plane(new Vector3(0, 1, 0), 0)
    const ray = new Raycaster()
    const pointOnPlane = new Vector3()
    const ground = {
    pitch: groundPitch * MathUtils.DEG2RAD,
    yaw: groundYaw * MathUtils.DEG2RAD
    }

    //建立投影平面
    plane.normal
    .set(0, Math.cos(ground.pitch), Math.sin(ground.pitch))
    .normalize()

    let point = generateCenter(plane);

    //地面中心到视窗的投影
    point = point.clone().project(camera);
    point = NDCToScreen(point, config.previewWindow.width, config.previewWindow.height)

    return point;
}

// 生成曲线 （wrapper） 返回曲线圆弧上的点（世界坐标）
function generateCurve(
  moveAngle = new Vector3(), // 下发的指令
  plane = new Plane(), // 圆所在平面
  speedRate,  //速度控制百分比
  angleRate,  //画多长的圆弧
) {
    
    return generateCurveBasedOnCircle(moveAngle, plane,speedRate,angleRate)
}

// 获得曲线的点
function getCurvePoints(
  moveAngle, //指令移动角（单位：角度）
  speedRate,  //速度控制百分比
  angleRate,  //画多长的圆弧
  groundPitch, //地面的上下摆动
  groundYaw,    //地面的左右摆动 固定为0
  cameraConfig,
) {
  const camera = buildCamera(cameraConfig)
  const plane = new Plane(new Vector3(0, 1, 0), 0)
  const ray = new Raycaster()
  const pointOnPlane = new Vector3()
  const ground = {
    pitch: groundPitch * MathUtils.DEG2RAD,
    yaw: groundYaw * MathUtils.DEG2RAD
  }

  //建立投影平面
  plane.normal
    .set(0, Math.cos(ground.pitch), Math.sin(ground.pitch))
    .normalize()

  const pointsObject = generateCurve(moveAngle,plane,speedRate,angleRate)
    
  //地面圆到主视窗的投影
  const pointsXYLeft = pointsObject.pointsLeft
    .map(p => p.clone().project(camera))
    .map(p => NDCToScreen(p, cameraConfig.previewWindow.width, cameraConfig.previewWindow.height))

  const pointsXY = pointsObject.points
    .map(p => p.clone().project(camera))
    .map(p => NDCToScreen(p, cameraConfig.previewWindow.width, cameraConfig.previewWindow.height))

  const pointsXYRight = pointsObject.pointsRight
    .map(p => p.clone().project(camera))
    .map(p => NDCToScreen(p, cameraConfig.previewWindow.width, cameraConfig.previewWindow.height))
    
//    return [pointsXYLeft,pointsXY,pointsXYRight]
    //window.stub.jsCallbackMethod("ssdsdsd");
    window.stub.jsCallbackMethod(JSON.stringify(pointsXYLeft));
  return { pointsLeft: pointsXYLeft, points: pointsXY, pointsRight: pointsXYRight }
}


function topPts(angle, speed, windowWidth, windowHeight) {
    //window.stub.jsCallbackMethod("111111111");
    var config = firstCameraConfig
    config.previewWindow.width = windowWidth;
    config.previewWindow.height = windowHeight;

    return getCurvePoints(angle,speed/100,0.1,0,0,config)
    
}

function bottomPts(angle, speed, windowWidth, windowHeight) {
    
    var config = secondCameraConfig
    config.previewWindow.width = windowWidth;
    config.previewWindow.height = windowHeight;

    return getCurvePoints(angle,speed/100,0.1,0,0,config)
    
}


/*(function(){
    topPts(30, 30, 300, 200);

})()*/

// // 获得曲线的点
// function getCurvePoints(
//   // mouseX, //鼠标在窗口的X坐标 px
//   // mouseY, //鼠标在窗口的Y坐标 px
//   moveAngle, //指令移动角（单位：角度）
//   speed,     //速度百分比（%）
//   groundPitch, //地面的上下摆动
//   groundYaw, //地面的左右摆动 固定为0
//   windowWidth = secondCameraConfig.previewWindow.width, // 窗口宽度
//   windowHeight = secondCameraConfig.previewWindow.height, // 窗口高度
//   rate = 0.1,         // 要绘制的曲线距离屏幕的比值
//   mainCameraConfig = firstCameraConfig,
//   controlCameraConfig = secondCameraConfig
// ) {

//   const cameraMain = buildCamera(mainCameraConfig)
//   const cameraControl = buildCamera(controlCameraConfig)
//   const plane = new Plane(new Vector3(0, 1, 0), 0)
//   const ray = new Raycaster()
//   const pointOnPlane = new Vector3()
//   // const mouse = getNDCFromMouse(mouseX, mouseY, windowWidth, windowHeight)
//   // const curveStart = new Vector3(0, 0, 0)
//   const ground = {
//     pitch: groundPitch * MathUtils.DEG2RAD,
//     yaw: groundYaw * MathUtils.DEG2RAD
//   }


//   //鼠标位置在世界坐标的投影
//   // plane.normal
//   //   .set(0, Math.cos(ground.pitch), Math.sin(ground.pitch))
//   //   .normalize()
//   // ray.setFromCamera(mouse, cameraControl)
//   // ray.ray.intersectPlane(plane, pointOnPlane)

//   //建立投影平面
//   plane.normal
//     .set(0, Math.cos(ground.pitch), Math.sin(ground.pitch))
//     .normalize()

//   const pointsObject = generateCurve(moveAngle, speed, plane, rate)

//   //地面圆到主视窗的投影
//   const pointsXYLeftMain = pointsObject.pointsLeft
//     .map(p => p.clone().project(cameraMain))
//     .map(p => NDCToScreen(p, mainCameraConfig.previewWindow.width, mainCameraConfig.previewWindow.height))

//   const pointsXYMain = pointsObject.points
//     .map(p => p.clone().project(cameraMain))
//     .map(p => NDCToScreen(p, mainCameraConfig.previewWindow.width, mainCameraConfig.previewWindow.height))

//   const pointsXYRightMain = pointsObject.pointsRight
//     .map(p => p.clone().project(cameraMain))
//     .map(p => NDCToScreen(p, mainCameraConfig.previewWindow.width, mainCameraConfig.previewWindow.height))

//   //地面圆到控制窗口的投影
//   const pointsXYLeftControl = pointsObject.pointsLeft
//     .map(p => p.clone().project(cameraControl))
//     .map(p => NDCToScreen(p, controlCameraConfig.previewWindow.width, controlCameraConfig.previewWindow.height))

//   const pointsXYControl = pointsObject.points
//     .map(p => p.clone().project(cameraControl))
//     .map(p => NDCToScreen(p, controlCameraConfig.previewWindow.width, controlCameraConfig.previewWindow.height))


//   const pointsXYRightControl = pointsObject.pointsRight
//     .map(p => p.clone().project(cameraControl))
//     .map(p => NDCToScreen(p, controlCameraConfig.previewWindow.width, controlCameraConfig.previewWindow.height))

//   return { pointsLeftMain: pointsXYLeftMain, pointsMain: pointsXYMain, pointsRightMain: pointsXYRightMain, pointsLeftControl: pointsXYLeftControl, pointsControl: pointsXYControl, pointsRightControl: pointsXYRightControl }
// }

export { topPts, bottomPts}