

//最大向前转速,单位: rpm   转/分
var MAX_FORWARD_SPEED = 50;

//最大倒退转速,单位: rpm   转/分
var MAX_BACKWARD_SPEED = 24;

//车轮直径，单位：米  ，为适应底盘加速过程，直径稍改小
var WHEEL_RADIUS = 0.0655;

//两轮地面间距，单位：米
var WHEEL_LENGTH = 0.373;

// 构建相机 具体参数见 config.js
function buildCamera (config) {
  const camera = new PerspectiveCamera(
    config.fov,
    config.videoWidth / config.videoHeight,
    0.1,
    100
  )
  if (config.cameraPosition) {
    const {
      x = camera.position.x,
      y = camera.position.y,
      z = camera.position.z
    } = config.cameraPosition
    camera.position.set(x,y,z)
  }
  if (config.cameraPose) {
    const { yaw = 0, pitch = 0 } = config.cameraPose
    camera.quaternion.setFromUnitVectors(
      new Vector3(0, 0, 1),
      new Vector3(
        Math.sin(yaw * MathUtils.DEG2RAD),
        Math.sin(-pitch * MathUtils.DEG2RAD),
        Math.cos(pitch * MathUtils.DEG2RAD)
      )
    )
  }
  camera.updateMatrix()
  camera.updateMatrixWorld()
  camera.updateProjectionMatrix()
  return camera
}

// 生成圆上的点
function generateCircle (
  origin, //圆心
  radius, // 半径
  segments = 30, // 圆的分段
  startAngle = 0, //  起始价角度
  addAngle = 2 * Math.PI // 圆弧的角度 分正负 // 逆时针为正
) {
  const points = []
  const segment = addAngle / segments
  for (let i = 0; i < segments; i++) {
    const angle = startAngle + i * segment
    const point = new Vector3(Math.cos(angle), 0, Math.sin(angle))
    point.setLength(radius)
    point.add(origin)
    points.push(point)
  }
  return points
}

//根据角速度和线速度求半径
function getRadius(angleDev = 0,baseSpeed = 0){

   angleDev = angleDev == 0 ? 0.01 : angleDev
   angleDev = angleDev == 90 ? 89.99 : angleDev

   const avgSpeed = baseSpeed * Math.cos(angleDev * Math.PI / 180);
   const kTheta = baseSpeed / 500;
   const vOutside = avgSpeed + kTheta * angleDev;
   const vInside = avgSpeed - kTheta * angleDev;

   const R = (WHEEL_LENGTH/2)*(vOutside + vInside)/(vOutside - vInside);

   return R * RADIUS_RATE;
}

function getAngle(rate = 0, radius = 1){
    rate = rate < 0.5 ? 0.5 : rate;
    const angle = 2 * rate / radius;
    return angle > 2 * Math.PI ? 2 * Math.PI : angle;

}

//根据指令求地面曲线基本参数，圆心、半径、其实角度（单位：弧度），增量角度（单位：弧度）
function getBasicParameter(
  moveAngle = 0,
  speedRate = 1,
  angleRate = 0
){
    let originPoint = new Vector3();
    let radius = -1;
    let startAngle = 0;
    let addAngle = 0;

    if (moveAngle >= 0 && moveAngle <= 90){
      const angleDev = moveAngle;
      radius = getRadius(angleDev,MAX_FORWARD_SPEED * speedRate);
      originPoint = new Vector3(-radius,0,0);
      startAngle = 0;
      addAngle = -getAngle(angleRate,radius);
    }
    else if (moveAngle > 90 && moveAngle <= 180 ){
      const angleDev = 180 - moveAngle;
      radius = getRadius(angleDev,MAX_FORWARD_SPEED * speedRate > MAX_BACKWARD_SPEED ? MAX_BACKWARD_SPEED : MAX_FORWARD_SPEED * speedRate);
      originPoint = new Vector3(-radius,0,0);
      startAngle = 0;
      addAngle = getAngle(angleRate,radius);
    }
    else if (moveAngle > 180 && moveAngle <= 270 ){
      const angleDev = moveAngle - 180;
      radius = getRadius(angleDev,MAX_FORWARD_SPEED * speedRate > MAX_BACKWARD_SPEED ? MAX_BACKWARD_SPEED : MAX_FORWARD_SPEED * speedRate);
      originPoint = new Vector3(radius,0,0);
      startAngle = Math.PI;
      addAngle = -getAngle(angleRate,radius);
    }
    else if (moveAngle > 270  && moveAngle < 360 ){
      const angleDev = 360 - moveAngle;
      radius = getRadius(angleDev,MAX_FORWARD_SPEED * speedRate);
      originPoint = new Vector3(radius,0,0);
      startAngle = Math.PI;
      addAngle = getAngle(angleRate,radius);
    }

    return [radius,originPoint,startAngle,addAngle];
}


function generateCenter(
  plane = new Plane(), // 点所在平面
)
{
    const point = new Vector3(0, 0, 0)
    const quaternion = new Quaternion()
    quaternion.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
    point.applyQuaternion(quaternion)
    return point;
}

// 在平面上生成圆弧曲线 返回曲线圆弧上的点（世界坐标）
function generateCurveBasedOnCircle (
  moveAngle = new Vector3(), // 终点 圆上任意一个点
  plane = new Plane(), // 圆所在平面
  speedRate = 1,
  angleRate
) {
  // const originPoint = getOriginPoint(start, moveAngle)  // 圆心
  // const radius = new THREE.Vector3().subVectors(start, originPoint).length() // 半径
  // const [startAngle, addAngle] = getAngles(originPoint, start, moveAngle) // 这里计算出来的addAngle startAngle为到鼠标点的圆弧角度

  const [radius,originPoint,startAngle,addAngle] = getBasicParameter(moveAngle,speedRate,angleRate);

  if (radius < 0 ) return null;

  //生成左面的圆
  const pointsLeft = generateCircle(
    originPoint,
    radius - WHEEL_LENGTH * WHEEL_RATE / 2,
    POINTS_COUNT,
    startAngle,
    addAngle
  )

  const quaternionLeft = new Quaternion()
  quaternionLeft.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
  for (const point of pointsLeft) {
    point.applyQuaternion(quaternionLeft)
  }

  //生成中间的圆
  const points = generateCircle(
    originPoint,
    radius,
    POINTS_COUNT,
    startAngle,
    addAngle
  )

  const quaternion = new Quaternion()
  quaternion.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
  for (const point of points) {
    point.applyQuaternion(quaternion)
  }

  //生成右面的圆
  const pointsRight = generateCircle(
    originPoint,
    radius + WHEEL_LENGTH * WHEEL_RATE / 2,
    POINTS_COUNT,
    startAngle,
    addAngle
  )
  const quaternionRight = new Quaternion()
  quaternionRight.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
  for (const point of pointsRight) {
    point.applyQuaternion(quaternionRight)
  }

  return {pointsLeft:pointsLeft,points:points,pointsRight:pointsRight}
}

// NDC 到屏幕坐标的转换
// NDC see refer/NDC.png
function NDCToScreen (
  point, // NDC坐标 范围为 -1到+1
  windowWidth, // 屏幕宽度 单位像素
  windowHeight // 屏幕高度 单位像素
) {
  return [
    parseInt(((point.x + 1) / 2) * windowWidth),
    parseInt((-(point.y - 1) / 2) * windowHeight)
  ]
}

// 从鼠标获得NDC坐标
function getNDCFromMouse (
  mouseX, // 鼠标X位置 单位像素
  mouseY, // 鼠标Y位置 单位像素
  windowWidth, // 屏幕宽度 单位像素
  windowHeight // 屏幕高度 单位像素
) {
  return new Vector2(
    (mouseX / windowWidth) * 2 - 1,
    -(mouseY / windowHeight) * 2 + 1
  )
}

// 在平面上生成中心圆弧曲线 基于圆弧返回箭头上的点（世界坐标）
function generateArrowBasedOnCircle(
  moveAngle = new Vector3(), // 终点 圆上任意一个点
  plane = new Plane(), // 圆所在平面
  speedRate = 1,
  angleRate
){
  const [radius,originPoint,startAngle,addAngle] = getBasicParameter(moveAngle,speedRate,angleRate);
  if (radius < 0 ) return null;
    //生成中间的圆
    const points = generateCircle(
      originPoint,
      radius,
      POINTS_COUNT,
      startAngle,
      addAngle
    )
    window.stub.jsCallbackMethod("333333333333--11111111111-22222222");
    const arrowTopPoints = getArrowPoint(originPoint,WHEEL_LENGTH * WHEEL_RATE / 3,points[points.length * 0.8])
    const quaternionArrowTop = new Quaternion()
    quaternionArrowTop.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
    for (const point of arrowTopPoints) {
      point.applyQuaternion(quaternionArrowTop)
    }
    window.stub.jsCallbackMethod("333333333333--11111111111-3333333333");
    const arrowMiddlePoints = getArrowPoint(originPoint,WHEEL_LENGTH * WHEEL_RATE / 3,points[points.length * 0.5])
    const quaternionArrowBottom = new Quaternion()
    quaternionArrowBottom.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
    for (const point of arrowMiddlePoints) {
      point.applyQuaternion(quaternionArrowBottom)
    }

    window.stub.jsCallbackMethod("333333333333--11111111111-4444444444");
    const arrowBottomPoints = getArrowPoint(originPoint,WHEEL_LENGTH * WHEEL_RATE / 3,points[points.length * 0.2])
    const quaternionArrowRight = new Quaternion()
    quaternionArrowRight.setFromUnitVectors(new Vector3(0, 1, 0), plane.normal)
    for (const point of arrowBottomPoints) {
      point.applyQuaternion(quaternionArrowRight)
    }
    window.stub.jsCallbackMethod("333333333333--11111111111-55555555555");
    return {arrowTopPoints, arrowMiddlePoints, arrowBottomPoints}
}

// 获取箭头顶点到左右两个控制点连线(世界坐标)
function getArrowPoint(originPoint,radius,vertexOrigin){
  const startAngle = Math.atan2(vertexOrigin.z,vertexOrigin.x - originPoint.x)

  const segmentLeft = Math.PI / 180 * (originPoint.x > 0 ? 210 : 30)
  const angleLeft = startAngle + segmentLeft
  const pointLeft = new Vector3(Math.cos(angleLeft), 0, Math.sin(angleLeft))
  pointLeft.setLength(radius)
  pointLeft.add(vertexOrigin)

  const segmentRight = Math.PI / 180 * (originPoint.x > 0 ? 330 : 150)
  const angleRight = startAngle + segmentRight
  const pointRight = new Vector3(Math.cos(angleRight), 0, Math.sin(angleRight))
  pointRight.setLength(radius)
  pointRight.add(vertexOrigin)

  const lineLeft = getLineForTowPoint(pointLeft,vertexOrigin)
  lineLeft.length = 14
  const lineRight = getLineForTowPoint(vertexOrigin,pointRight)

  return [...lineLeft,...lineRight]
}

// 获取两点连线所生成的点(世界坐标)
function getLineForTowPoint(start, end, segments = 15){

  let points = []
  const segment = Math.abs(start.x - end.x) / segments
  for(let i = 0; i < segments; i++){
    let x = start.x + (start.x > end.x ? -segment * i : segment * i)
    let z = (x - start.x) / (end.x - start.x) * (end.z - start.z) + start.z
    const point = new Vector3(x, 0, z)
    points.push(point)
  }
  return points
}